@@ -1,11 +1,9 @@
-# poly.rb -- polynomial-related stuff; poly.scm --> poly.rb -*- snd-ruby -*-
+# poly.rb -- polynomial-related stuff; poly.scm --> poly.rb
 
 # Translator: Michael Scholz <mi-scholz@users.sourceforge.net>
-# Created: Sat Apr 09 23:55:07 CEST 2005
-# Changed: Wed Oct 14 23:12:00 CEST 2009
+# Created: 05/04/09 23:55:07
+# Changed: 17/11/30 22:57:04
 
-# Commentary: (see poly.scm)
-#
 # class Complex
 #  to_f
 #  to_f_or_c
@@ -49,15 +47,16 @@
 # poly_derivative(obj)
 # poly_gcd(obj1, obj2)
 # poly_roots(obj)
-#
-# Code:
 
-require "complex"
 require "clm"
 require "mix"
+include Math
 
 class Complex
-  attr_writer :real, :imag
+  # XXX: attr_writer :real, :imag
+  #      Doesn't work any longer.
+  #      Complex objects are now frozen objects.
+  #      (Thu Nov 30 21:29:10 CET 2017)
   with_silence do
     def to_f
       self.real.to_f
@@ -126,7 +125,7 @@ class Poly < Vec
       m = Poly.new(len, 0.0)
       self.each_with_index do |val1, i|
         other.each_with_index do |val2, j|
-          m[i + j] += val1 * val2
+          m[i + j] = m[i + j] + val1 * val2
         end
       end
       m
@@ -154,20 +153,30 @@ class Poly < Vec
         nv = other.length - 1
         (n - nv).downto(0) do |i|
           q[i] = r[nv + i] / other[nv]
-          (nv + i - 1).downto(i) do |j| r[j] = r[j] - q[i] * other[j - i] end
+          (nv + i - 1).downto(i) do |j|
+            r[j] = r[j] - q[i] * other[j - i]
+          end
+        end
+        nv.upto(n) do |i|
+          r[i] = 0.0
         end
-        nv.upto(n) do |i| r[i] = 0.0 end
         [q, r]
       end
     end
   end
   alias / poly_div
-  # poly(-1.0, 0.0, 1.0) / poly(1.0, 1.0) ==> [poly(-1.0, 1.0, 0.0),       poly(0.0, 0.0, 0.0)]
-  # poly(-15, -32, -3, 2) / poly(-5, 1)   ==> [poly(3.0, 7.0, 2.0, 0.0),   poly(0.0, 0.0, 0.0, 0.0)]
-  # poly(-15, -32, -3, 2) / poly(3, 1)    ==> [poly(-5.0, -9.0, 2.0, 0.0), poly(0.0, 0.0, 0.0, 0.0)]
-  # poly(-15, -32, -3, 2) / poly(0.5, 1) ==> [poly(-30.0, -4.0, 2.0, 0.0), poly(0.0, 0.0, 0.0, 0.0)]
-  # poly(-15, -32, -3, 2) / poly(3, 7, 2) ==> [poly(-5.0, 1.0, 0.0, 0.0),  poly(0.0, 0.0, 0.0, 0.0)]
-  # poly(-15, -32, -3, 2) / 2.0           ==> [poly(-7.5, -16.0, -1.5, 1.0), poly(0.0)]
+  # poly(-1.0, 0.0, 1.0) / poly(1.0, 1.0)
+  #   ==> [poly(-1.0, 1.0, 0.0),       poly(0.0, 0.0, 0.0)]
+  # poly(-15, -32, -3, 2) / poly(-5, 1)
+  #   ==> [poly(3.0, 7.0, 2.0, 0.0),   poly(0.0, 0.0, 0.0, 0.0)]
+  # poly(-15, -32, -3, 2) / poly(3, 1)
+  #   ==> [poly(-5.0, -9.0, 2.0, 0.0), poly(0.0, 0.0, 0.0, 0.0)]
+  # poly(-15, -32, -3, 2) / poly(0.5, 1)
+  #   ==> [poly(-30.0, -4.0, 2.0, 0.0), poly(0.0, 0.0, 0.0, 0.0)]
+  # poly(-15, -32, -3, 2) / poly(3, 7, 2)
+  #   ==> [poly(-5.0, 1.0, 0.0, 0.0),  poly(0.0, 0.0, 0.0, 0.0)]
+  # poly(-15, -32, -3, 2) / 2.0
+  #   ==> [poly(-7.5, -16.0, -1.5, 1.0), poly(0.0)]
 
   def derivative
     len = self.length - 1
@@ -181,24 +190,26 @@ class Poly < Vec
   end
   # poly(0.5, 1.0, 2.0, 4.0).derivative ==> poly(1.0, 4.0, 12.0)
 
-  include Mixer_matrix
   def resultant(other)
     m = self.length
     m1 = m - 1
     n = other.length
     n1 = n - 1
-    mat = make_mixer(n1 + m1)
+    d = n1 + m1
+    mat = Array.new(d) do
+      Vct.new(d, 0.0)
+    end
     n1.times do |i|
       m.times do |j|
-        mixer_set!(mat, i, i + j, self[m1 - j])
+        mat[i][i + j] = self[m1 - j]
       end
     end
     m1.times do |i|
       n.times do |j|
-        mixer_set!(mat, i + n1, i + j, other[n1 - j])
+        mat[i + n1][i + j] = other[n1 - j]
       end
     end
-    mixer_determinant(mat)
+    determinant(mat)
   end
   # poly(-1, 0, 1).resultant([1, -2, 1]) ==> 0.0
   # poly(-1, 0, 2).resultant([1, -2, 1]) ==> 1.0
@@ -210,17 +221,24 @@ class Poly < Vec
   end
   # poly(-1, 0, 1).discriminant ==> -4.0
   # poly(1, -2, 1).discriminant ==>  0.0
-  # (poly(-1, 1) * poly(-1, 1) * poly(3, 1)).reduce.discriminant              ==> 0.0
-  # (poly(-1, 1) * poly(-1, 1) * poly(3, 1) * poly(2, 1)).reduce.discriminant ==> 0.0
-  # (poly(1, 1) * poly(-1, 1) * poly(3, 1) * poly(2, 1)).reduce.discriminant  ==> 2304.0
-  # (poly(1, 1) * poly(-1, 1) * poly(3, 1) * poly(3, 1)).reduce.discriminant  ==> 0.0
+  # (poly(-1, 1) * poly(-1, 1) * poly(3, 1)).reduce.discriminant
+  #   ==> 0.0
+  # (poly(-1, 1) * poly(-1, 1) * poly(3, 1) * poly(2, 1)).reduce.discriminant
+  #   ==> 0.0
+  # (poly(1, 1) * poly(-1, 1) * poly(3, 1) * poly(2, 1)).reduce.discriminant
+  #   ==> 2304.0
+  # (poly(1, 1) * poly(-1, 1) * poly(3, 1) * poly(3, 1)).reduce.discriminant
+  #   ==> 0.0
   
   def gcd(other)
-    assert_type((array?(other) or vct?(other)), other, 0, "a poly, a vct or an array")
+    assert_type((array?(other) or vct?(other)), other, 0,
+                "a poly, a vct or an array")
     if self.length < other.length
       poly(0.0)
     else
-      qr = self.poly_div(other).map do |m| m.reduce end
+      qr = self.poly_div(other).map do |m|
+        m.reduce
+      end
       if qr[1].length == 1
         if qr[1][0].zero?
           Poly(other)
@@ -232,26 +250,37 @@ class Poly < Vec
       end
     end
   end
-  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(2, 1))               ==> poly(2.0, 1.0)
-  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(3, 1))               ==> poly(0.0)
-  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(-3, 1))              ==> poly(-3.0, 1.0)
-  # (poly(8, 1) * poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(-3, 1)) ==> poly(-3.0, 1.0)
-  # (poly(8, 1) * poly(2, 1) * poly(-3, 1)).reduce.gcd((poly(8, 1) * poly(-3, 1)).reduce)
-  #                                                                 ==> poly(-24.0, 5.0, 1.0)
-  # poly(-1, 0, 1).gcd(poly(2, -2, -1, 1))                          ==> poly(0.0)
-  # poly(2, -2, -1, 1).gcd(poly(-1, 0, 1))                          ==> poly(1.0, -1.0)
-  # poly(2, -2, -1, 1).gcd(poly(-2.5, 1))                           ==> poly(0.0)
+  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(2, 1))
+  #   ==> poly(2.0, 1.0)
+  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(3, 1))
+  #   ==> poly(0.0)
+  # (poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(-3, 1))
+  #   ==> poly(-3.0, 1.0)
+  # (poly(8, 1) * poly(2, 1) * poly(-3, 1)).reduce.gcd(poly(-3, 1))
+  #   ==> poly(-3.0, 1.0)
+  # (poly(8, 1) * poly(2, 1) *
+  #  poly(-3, 1)).reduce.gcd((poly(8, 1) * poly(-3, 1)).reduce)
+  #   ==> poly(-24.0, 5.0, 1.0)
+  # poly(-1, 0, 1).gcd(poly(2, -2, -1, 1))
+  #   ==> poly(0.0)
+  # poly(2, -2, -1, 1).gcd(poly(-1, 0, 1))
+  #   ==> poly(1.0, -1.0)
+  # poly(2, -2, -1, 1).gcd(poly(-2.5, 1))
+  #   ==> poly(0.0)
 
   def roots
     rts = poly()
-    if (deg = self.length - 1).zero?
+    deg = self.length - 1
+    if deg.zero?
       rts
     else
       if self[0].zero?
         if deg == 1
           poly(0.0)
         else
-          Poly.new(deg) do |i| self[i + 1] end.roots.unshift(0.0)
+          Poly.new(deg) do |i|
+            self[i + 1]
+          end.roots.unshift(0.0)
         end
       else
         if deg == 1
@@ -260,14 +289,21 @@ class Poly < Vec
           if deg == 2
             quadratic_root(self[2], self[1], self[0])
           else
-            if deg == 3 and (rts = cubic_root(self[3], self[2], self[1], self[0]))
+            if deg == 3 and
+               (rts = cubic_root(self[3], self[2], self[1], self[0]))
               rts
             else
-              if deg == 4 and (rts = quartic_root(self[4], self[3], self[2], self[1], self[0]))
+              if deg == 4 and
+                 (rts = quartic_root(self[4], self[3],
+                                     self[2], self[1], self[0]))
                 rts
               else
                 ones = 0
-                1.upto(deg) do |i| if self[i].nonzero? then ones += 1 end end
+                1.upto(deg) do |i|
+                  if self[i].nonzero?
+                    ones += 1
+                  end
+                end
                 if ones == 1
                   nth_root(self[deg], self[0], deg)
                 else
@@ -284,7 +320,10 @@ class Poly < Vec
                         self[deg / 3].nonzero? and
                         self[(deg * 2) / 3].nonzero?
                       n = deg / 3
-                      poly(self[0],self[deg / 3],self[(deg * 2) / 3],self[deg]).roots.each do |qr|
+                      poly(self[0],
+                           self[deg / 3],
+                           self[(deg * 2) / 3],
+                           self[deg]).roots.each do |qr|
                         rts.push(*nth_root(1.0, -qr, n.to_f))
                       end
                       rts
@@ -297,9 +336,13 @@ class Poly < Vec
                       v = q.eval(x)
                       m = v.abs * v.abs
                       20.times do # until c_g?
-                        if (dx = v / qp.eval(x)).abs <= Poly_roots_epsilon then break end
+                        if (dx = v / qp.eval(x)).abs <= Poly_roots_epsilon
+                          break
+                        end
                         20.times do
-                          if dx.abs <= Poly_roots_epsilon then break end
+                          if dx.abs <= Poly_roots_epsilon
+                            break
+                          end
                           y = x - dx
                           v1 = q.eval(y)
                           if (m1 = v1.abs * v1.abs) < m
@@ -308,7 +351,7 @@ class Poly < Vec
                             m = m1
                             break
                           else
-                            dx /= 4.0 # /: slash to fool Emacs' indentation commands
+                            dx /= 4.0
                           end
                         end
                       end
@@ -321,7 +364,12 @@ class Poly < Vec
                         q = q.poly_div(poly(x.abs, 0.0, 1.0))
                         n -= 2
                       end
-                      rts = ((n > 0) ? q.car.reduce.roots : poly()) << x.to_f_or_c
+                      rts = if n > 0
+                              q.car.reduce.roots
+                            else
+                              poly()
+                            end
+                      rts << x.to_f_or_c
                       rts
                     end
                   end
@@ -336,11 +384,63 @@ class Poly < Vec
   
   def eval(x)
     sum = self.last
-    self.reverse[1..-1].each do |val| sum = sum * x + val end
+    self.reverse[1..-1].each do |val|
+      sum = sum * x + val
+    end
     sum
   end
 
   private
+  def submatrix(mx, row, col)
+    nmx = Array.new(mx.length - 1) do
+      Vct.new(mx.length - 1, 0.0)
+    end
+    ni = 0
+    mx.length.times do |i|
+      if i != row
+        nj = 0
+        mx.length.times do |j|
+          if j != col
+            nmx[ni][nj] = mx[i][j]
+            nj += 1
+          end
+        end
+        ni += 1
+      end
+    end
+    nmx
+  end
+
+  def determinant(mx)
+    if mx.length == 1
+      mx[0][0]
+    else
+      if mx.length == 2
+        mx[0][0] * mx[1][1] - mx[0][1] * mx[1][0]
+      else
+        if mx.length == 3
+          ((mx[0][0] * mx[1][1] * mx[2][2] +
+            mx[0][1] * mx[1][2] * mx[2][0] +
+            mx[0][2] * mx[1][0] * mx[2][1]) -
+           (mx[0][0] * mx[1][2] * mx[2][1] +
+            mx[0][1] * mx[1][0] * mx[2][2] +
+            mx[0][2] * mx[1][1] * mx[2][0]))
+        else
+          sum = 0.0
+          sign = 1
+          mx.length.times do |i|
+            mult = mx[0][i]
+            if mult != 0.0
+              sum = sum + sign * mult * determinant(submatrix(mx, 0, i))
+            end
+            sign = -sign
+          end
+          sum
+        end
+      end
+    end
+  end
+
   # ax + b
   def linear_root(a, b)
     poly(-b / a)
@@ -372,9 +472,13 @@ class Poly < Vec
         s2 = r2 * exp(j * incr)
         z1 = simplify_complex((s1 + s2) - (a2 / 3))
         if pl.eval(z1).abs < Poly_roots_epsilon
-          z2 = simplify_complex((-0.5 * (s1 + s2)) + (a2 / -3) + ((s1 - s2) * 0.5 * sqrt3))
+          z2 = simplify_complex((-0.5 * (s1 + s2)) +
+                                (a2 / -3) +
+                                ((s1 - s2) * 0.5 * sqrt3))
           if pl.eval(z2).abs < Poly_roots_epsilon
-            z3 = simplify_complex((-0.5 * (s1 + s2)) + (a2 / -3) + ((s1 - s2) * -0.5 * sqrt3))
+            z3 = simplify_complex((-0.5 * (s1 + s2)) +
+                                  (a2 / -3) +
+                                  ((s1 - s2) * -0.5 * sqrt3))
             if pl.eval(z3).abs < Poly_roots_epsilon
               return poly(z1, z2, z3)
             end
@@ -399,16 +503,21 @@ class Poly < Vec
       yroot.each do |y1|
         r = sqrt((0.25 * a3 * a3) + (-a2 + y1))
         dd = if r.zero?
-              sqrt((0.75 * a3 * a3) + (-2 * a2) + (2 * sqrt(y1 * y1 - 4 * a0)))
+              sqrt((0.75 * a3 * a3) +
+                   (-2 * a2) +
+                   (2 * sqrt(y1 * y1 - 4 * a0)))
             else
               sqrt((0.75 * a3 * a3) + (-2 * a2) + (-(r * r)) +
                    (0.25 * ((4 * a3 * a2) + (-8 * a1) + (-(a3 * a3 * a3)))) / r)
             end
         ee = if r.zero?
-              sqrt((0.75 * a3 * a3) + (-2 * a2) + (-2 * sqrt((y1 * y1) - (4 * a0))))
+              sqrt((0.75 * a3 * a3) +
+                   (-2 * a2) +
+                   (-2 * sqrt((y1 * y1) - (4 * a0))))
             else
               sqrt((0.75 * a3 * a3) + (-2 * a2) + (-(r * r)) +
-                   (-0.25 * ((4 * a3 * a2) + (-8 * a1) + (-(a3 * a3 * a3)))) / r)
+                   (-0.25 *
+                    ((4 * a3 * a2) + (-8 * a1) + (-(a3 * a3 * a3)))) / r)
             end
         z1 = (-0.25 * a3) + ( 0.5 * r) + ( 0.5 * dd)
         z2 = (-0.25 * a3) + ( 0.5 * r) + (-0.5 * dd)
@@ -427,7 +536,9 @@ class Poly < Vec
     n = (-b / a) ** (1.0 / deg)
     incr = (TWO_PI * Complex::I) / deg
     rts = poly()
-    deg.to_i.times do |i| rts.unshift(simplify_complex(exp(i * incr) * n)) end
+    deg.to_i.times do |i|
+      rts.unshift(simplify_complex(exp(i * incr) * n))
+    end
     rts
   end
 
@@ -436,7 +547,11 @@ class Poly < Vec
     if a.imag.abs < Poly_roots_epsilon2
       (a.real.abs < Poly_roots_epsilon2) ? 0.0 : a.real.to_f
     else
-      if a.real.abs < Poly_roots_epsilon2 then a.real = 0.0 end
+      if a.real.abs < Poly_roots_epsilon2
+        # XXX: a.real = 0.0
+        #      Doesn't work any longer (see above, class Complex).
+        a = Complex(0.0, a.imag)
+      end
       a
     end
   end
@@ -507,9 +622,11 @@ class Vct
 end
 
 def Poly(obj)
-  if obj.nil? then obj = [] end
+  if obj.nil?
+    obj = []
+  end
   assert_type(obj.respond_to?(:to_poly), obj, 0,
-              "an object containing method 'to_poly' (Vct, String, Array and subclasses)")
+              "an object containing method 'to_poly'")
   obj.to_poly
 end
 
@@ -532,52 +649,62 @@ def poly(*vals)
 end
 
 def poly_reduce(obj)
-  assert_type(obj.respond_to?(:to_poly), obj, 0, "an object containing method 'to_poly'")
+  assert_type(obj.respond_to?(:to_poly), obj, 0,
+              "an object containing method 'to_poly'")
   Poly(obj).reduce
 end
 
 def poly_add(obj1, obj2)
   if number?(obj1)
-    assert_type(obj2.respond_to?(:to_poly), obj2, 1, "an object containing method 'to_poly'")
+    assert_type(obj2.respond_to?(:to_poly), obj2, 1,
+                "an object containing method 'to_poly'")
     Float(obj1) + Poly(obj2)
   else
-    assert_type(obj1.respond_to?(:to_poly), obj1, 0, "an object containing method 'to_poly'")
+    assert_type(obj1.respond_to?(:to_poly), obj1, 0,
+                "an object containing method 'to_poly'")
     Poly(obj1) + obj2
   end
 end
 
 def poly_multiply(obj1, obj2)
   if number?(obj1)
-    assert_type(obj2.respond_to?(:to_poly), obj2, 1, "an object containing method 'to_poly'")
+    assert_type(obj2.respond_to?(:to_poly), obj2, 1,
+                "an object containing method 'to_poly'")
     Float(obj1) * Poly(obj2)
   else
-    assert_type(obj1.respond_to?(:to_poly), obj1, 0, "an object containing method 'to_poly'")
+    assert_type(obj1.respond_to?(:to_poly), obj1, 0,
+                "an object containing method 'to_poly'")
     Poly(obj1) * obj2
   end
 end
 
 def poly_div(obj1, obj2)
   if number?(obj1)
-    assert_type(obj2.respond_to?(:to_poly), obj2, 1, "an object containing method 'to_poly'")
+    assert_type(obj2.respond_to?(:to_poly), obj2, 1,
+                "an object containing method 'to_poly'")
     Float(obj1) / Poly(obj2)
   else
-    assert_type(obj1.respond_to?(:to_poly), obj1, 0, "an object containing method 'to_poly'")
+    assert_type(obj1.respond_to?(:to_poly), obj1, 0,
+                "an object containing method 'to_poly'")
     Poly(obj1) / obj2
   end
 end
 
 def poly_derivative(obj)
-  assert_type(obj.respond_to?(:to_poly), obj, 0, "an object containing method 'to_poly'")
+  assert_type(obj.respond_to?(:to_poly), obj, 0,
+              "an object containing method 'to_poly'")
   Poly(obj).derivative
 end
 
 def poly_gcd(obj1, obj2)
-  assert_type(obj.respond_to?(:to_poly), obj, 0, "an object containing method 'to_poly'")
+  assert_type(obj.respond_to?(:to_poly), obj, 0,
+              "an object containing method 'to_poly'")
   Poly(obj1).gcd(obj2)
 end
 
 def poly_roots(obj)
-  assert_type(obj.respond_to?(:to_poly), obj, 0, "an object containing method 'to_poly'")
+  assert_type(obj.respond_to?(:to_poly), obj, 0,
+              "an object containing method 'to_poly'")
   Poly(obj).roots
 end
 
