@@ -115,4 +115,7 @@
 
 ;(do ((i 0 (+ i 1))) ((= i 1000)) (test-chars))
 
+(when (> (*s7* 'profile) 0)
+  (show-profile 200))
 (exit)
+
